@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2011, Oracle and/or its affiliates
-   Copyright (c) 2010, 2020, MariaDB
+   Copyright (c) 2010, 2015, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -250,7 +250,7 @@ int init_io_cache(IO_CACHE *info, File file, size_t cachesize,
 	  info->write_buffer= info->buffer + cachesize;
         else
           info->write_buffer= info->buffer;
-	info->alloced_buffer= buffer_block;
+	info->alloced_buffer= 1;
 	break;					/* Enough memory found */
       }
       if (cachesize == min_cache)
@@ -324,14 +324,14 @@ int init_slave_io_cache(IO_CACHE *master, IO_CACHE *slave)
   DBUG_ASSERT(!master->share);
   DBUG_ASSERT(master->alloced_buffer);
 
-  if (!(slave_buf= (uchar*)my_malloc(master->alloced_buffer, MYF(0))))
+  if (!(slave_buf= (uchar*)my_malloc(master->buffer_length, MYF(0))))
   {
     return 1;
   }
   memcpy(slave, master, sizeof(IO_CACHE));
   slave->buffer= slave_buf;
 
-  memcpy(slave->buffer, master->buffer, master->alloced_buffer);
+  memcpy(slave->buffer, master->buffer, master->buffer_length);
   slave->read_pos= slave->buffer + (master->read_pos - master->buffer);
   slave->read_end= slave->buffer + (master->read_end - master->buffer);
 
@@ -496,10 +496,20 @@ my_bool reinit_io_cache(IO_CACHE *info, enum cache_type type,
   {
     /*
       If we change from WRITE_CACHE to READ_CACHE, assume that everything
-      after the current positions should be ignored
+      after the current positions should be ignored. In other cases we
+      update end_of_file as it may have changed since last init.
     */
-    if (info->type == WRITE_CACHE && type == READ_CACHE)
-      info->end_of_file=my_b_tell(info);
+    if (type == READ_CACHE)
+    {
+      if (info->type == WRITE_CACHE)
+	info->end_of_file= my_b_tell(info);
+      else
+      {
+        if (!(info->myflags & MY_ENCRYPT))
+          info->end_of_file= mysql_file_seek(info->file, 0L,
+                                             MY_SEEK_END, MYF(0));
+      }
+    }
     /* flush cache if we want to reuse it */
     if (!clear_cache && my_b_flush_io_cache(info,1))
       DBUG_RETURN(1);
@@ -804,8 +814,7 @@ int _my_b_cache_read(IO_CACHE *info, uchar *Buffer, size_t Count)
   info->read_pos=info->buffer+Count;
   info->read_end=info->buffer+length;
   info->pos_in_file=pos_in_file;
-  if (Count)
-    memcpy(Buffer, info->buffer, Count);
+  memcpy(Buffer, info->buffer, Count);
   DBUG_RETURN(0);
 }
 
@@ -1306,8 +1315,7 @@ static int _my_b_cache_read_r(IO_CACHE *cache, uchar *Buffer, size_t Count)
       DBUG_RETURN(1);
     }
     cnt= (len > Count) ? Count : len;
-    if (cnt)
-      memcpy(Buffer, cache->read_pos, cnt);
+    memcpy(Buffer, cache->read_pos, cnt);
     Count -= cnt;
     Buffer+= cnt;
     left_length+= cnt;
@@ -1548,8 +1556,7 @@ int _my_b_async_read(IO_CACHE *info, uchar *Buffer, size_t Count)
     if (info->aio_result.result.aio_errno)
     {
       if (info->myflags & MY_WME)
-	my_error(EE_READ, MYF(ME_BELL+ME_WAITTANG),
-		 my_filename(info->file),
+	my_error(EE_READ, MYF(ME_BELL), my_filename(info->file),
 		 info->aio_result.result.aio_errno);
       my_errno=info->aio_result.result.aio_errno;
       info->error= -1;
@@ -1603,7 +1610,7 @@ int _my_b_async_read(IO_CACHE *info, uchar *Buffer, size_t Count)
     Buffer+=length;
     Count-=length;
     left_length+=length;
-    info->read_end=info->read_pos+read_length;
+    info->read_end=info->rc_pos+read_length;
     info->read_pos+=length;
   }
   else
@@ -1642,8 +1649,7 @@ int _my_b_async_read(IO_CACHE *info, uchar *Buffer, size_t Count)
       if (Count != use_length)
       {					/* Didn't find hole block */
 	if (info->myflags & (MY_WME | MY_FAE | MY_FNABP) && Count != org_Count)
-	  my_error(EE_EOFERR, MYF(ME_BELL+ME_WAITTANG),
-		   my_filename(info->file),my_errno);
+	  my_error(EE_EOFERR, MYF(ME_BELL), my_filename(info->file), my_errno);
 	info->error=(int) (read_length+left_length);
 	return 1;
       }
