@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2012, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2016, MariaDB
+   Copyright (c) 2009, 2020, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -453,7 +453,8 @@ static bool extract_date_time(THD *thd, DATE_TIME_FORMAT *format,
       {
         ErrConvString err(val_begin, length, &my_charset_bin);
         make_truncated_value_warning(thd, Sql_condition::WARN_LEVEL_WARN,
-                                     &err, cached_timestamp_type, 0, NullS);
+                                     &err, cached_timestamp_type,
+                                     nullptr, nullptr, nullptr);
 	break;
       }
     } while (++val != val_end);
@@ -474,7 +475,7 @@ err:
 
 
 /**
-  Create a formated date/time value in a string.
+  Create a formatted date/time value in a string.
 */
 
 static bool make_date_time(const LEX_CSTRING &format, MYSQL_TIME *l_time,
@@ -1051,7 +1052,7 @@ uint week_mode(uint mode)
 	a date at start of january) In this case one can get 53 for the
 	first week of next year.  This flag ensures that the week is
 	relevant for the given year. Note that this flag is only
-	releveant if WEEK_JANUARY is not set.
+	relevant if WEEK_JANUARY is not set.
 
 			  If set	 Week is in range 1-53.
 
@@ -1353,7 +1354,7 @@ bool get_interval_value(THD *thd, Item *args,
     if (!(res= args->val_str_ascii(&str_value)))
       return (1);
 
-    /* record negative intervalls in interval->neg */
+    /* record negative intervals in interval->neg */
     str=res->ptr();
     cs= res->charset();
     const char *end=str+res->length();
@@ -1496,7 +1497,7 @@ bool Item_func_from_days::get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzz
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for local
+    Converts current time in my_time_t to MYSQL_TIME representation for local
     time zone. Defines time zone (local) used for whole CURDATE function.
 */
 void Item_func_curdate_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1507,7 +1508,7 @@ void Item_func_curdate_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for UTC
+    Converts current time in my_time_t to MYSQL_TIME representation for UTC
     time zone. Defines time zone (UTC) used for whole UTC_DATE function.
 */
 void Item_func_curdate_utc::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1580,12 +1581,12 @@ static void set_sec_part(ulong sec_part, MYSQL_TIME *ltime, Item *item)
   {
     ltime->second_part= sec_part;
     if (item->decimals < TIME_SECOND_PART_DIGITS)
-      my_time_trunc(ltime, item->decimals);
+      my_datetime_trunc(ltime, item->decimals);
   }
 }
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for local
+    Converts current time in my_time_t to MYSQL_TIME representation for local
     time zone. Defines time zone (local) used for whole CURTIME function.
 */
 void Item_func_curtime_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1599,7 +1600,7 @@ void Item_func_curtime_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for UTC
+    Converts current time in my_time_t to MYSQL_TIME representation for UTC
     time zone. Defines time zone (UTC) used for whole UTC_TIME function.
 */
 void Item_func_curtime_utc::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1653,7 +1654,7 @@ int Item_func_now_local::save_in_field(Field *field, bool no_conversions)
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for local
+    Converts current time in my_time_t to MYSQL_TIME representation for local
     time zone. Defines time zone (local) used for whole NOW function.
 */
 void Item_func_now_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1665,7 +1666,7 @@ void Item_func_now_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for UTC
+    Converts current time in my_time_t to MYSQL_TIME representation for UTC
     time zone. Defines time zone (UTC) used for whole UTC_TIMESTAMP function.
 */
 void Item_func_now_utc::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1695,7 +1696,7 @@ bool Item_func_now::get_date(THD *thd, MYSQL_TIME *res,
 
 
 /**
-    Converts current time in my_time_t to MYSQL_TIME represenatation for local
+    Converts current time in my_time_t to MYSQL_TIME representation for local
     time zone. Defines time zone (local) used for whole SYSDATE function.
 */
 void Item_func_sysdate_local::store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)
@@ -1981,7 +1982,7 @@ bool Item_func_convert_tz::get_date(THD *thd, MYSQL_TIME *ltime,
     uint not_used;
     my_time_tmp= from_tz->TIME_to_gmt_sec(ltime, &not_used);
     ulong sec_part= ltime->second_part;
-    /* my_time_tmp is guranteed to be in the allowed range */
+    /* my_time_tmp is guaranteed to be in the allowed range */
     if (my_time_tmp)
       to_tz->gmt_sec_to_TIME(ltime, my_time_tmp);
     /* we rely on the fact that no timezone conversion can change sec_part */
@@ -2402,7 +2403,7 @@ void Item_char_typecast::fix_length_and_dec_internal(CHARSET_INFO *from_cs)
   uint32 char_length;
   /* 
      We always force character set conversion if cast_cs
-     is a multi-byte character set. It garantees that the
+     is a multi-byte character set. It guarantees that the
      result of CAST is a well-formed string.
      For single-byte character sets we allow just to copy
      from the argument. A single-byte character sets string
