@@ -14,7 +14,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
-#include <my_global.h> // For HAVE_REPLICATION
+#include "mariadb.h" // For HAVE_REPLICATION
 #include "sql_priv.h"
 #include <my_dir.h>
 #include "rpl_mi.h"
@@ -28,7 +28,7 @@
 
 static void init_master_log_pos(Master_info* mi);
 
-Master_info::Master_info(LEX_STRING *connection_name_arg,
+Master_info::Master_info(LEX_CSTRING *connection_name_arg,
                          bool is_slave_recovery)
   :Slave_reporting_capability("I/O"),
    ssl(0), ssl_verify_server_cert(1), fd(-1), io_thd(0), 
@@ -42,8 +42,10 @@ Master_info::Master_info(LEX_STRING *connection_name_arg,
    using_gtid(USE_GTID_NO), events_queued_since_last_gtid(0),
    gtid_reconnect_event_skip_count(0), gtid_event_seen(false),
    in_start_all_slaves(0), in_stop_all_slaves(0), in_flush_all_relay_logs(0),
-   users(0), killed(0)
+   users(0), killed(0),
+   total_ddl_groups(0), total_non_trans_groups(0), total_trans_groups(0)
 {
+  char *tmp;
   host[0] = 0; user[0] = 0; password[0] = 0;
   ssl_ca[0]= 0; ssl_capath[0]= 0; ssl_cert[0]= 0;
   ssl_cipher[0]= 0; ssl_key[0]= 0;
@@ -55,16 +57,14 @@ Master_info::Master_info(LEX_STRING *connection_name_arg,
   */
   connection_name.length= cmp_connection_name.length=
     connection_name_arg->length;
-  if ((connection_name.str= (char*) my_malloc(connection_name_arg->length*2+2,
-                                              MYF(MY_WME))))
+  if ((connection_name.str= tmp= (char*)
+       my_malloc(connection_name_arg->length*2+2, MYF(MY_WME))))
   {
-    cmp_connection_name.str= (connection_name.str +
-                              connection_name_arg->length+1);
-    strmake(connection_name.str, connection_name_arg->str,
-            connection_name.length);
-    memcpy(cmp_connection_name.str, connection_name_arg->str,
-           connection_name.length+1);
-    my_casedn_str(system_charset_info, cmp_connection_name.str);
+    strmake(tmp, connection_name_arg->str, connection_name.length);
+    tmp+= connection_name_arg->length+1;
+    cmp_connection_name.str= tmp;
+    memcpy(tmp, connection_name_arg->str, connection_name.length+1);
+    my_casedn_str(system_charset_info, tmp);
   }
   /*
     When MySQL restarted, all Rpl_filter settings which aren't in the my.cnf
@@ -115,7 +115,7 @@ void Master_info::wait_until_free()
 Master_info::~Master_info()
 {
   wait_until_free();
-  my_free(connection_name.str);
+  my_free(const_cast<char*>(connection_name.str));
   delete_dynamic(&ignore_server_ids);
   mysql_mutex_destroy(&run_lock);
   mysql_mutex_destroy(&data_lock);
@@ -170,8 +170,6 @@ void Master_info::clear_in_memory_info(bool all)
   {
     port= MYSQL_PORT;
     host[0] = 0; user[0] = 0; password[0] = 0;
-    domain_id_filter.clear_ids();
-    reset_dynamic(&ignore_server_ids);
   }
 }
 
@@ -671,7 +669,7 @@ file '%s')", fname);
   mi->rli.is_relay_log_recovery= FALSE;
   // now change cache READ -> WRITE - must do this before flush_master_info
   reinit_io_cache(&mi->file, WRITE_CACHE, 0L, 0, 1);
-  if ((error= MY_TEST(flush_master_info(mi, TRUE, TRUE))))
+  if (unlikely((error= MY_TEST(flush_master_info(mi, TRUE, TRUE)))))
     sql_print_error("Failed to flush master info file");
   mysql_mutex_unlock(&mi->data_lock);
   DBUG_RETURN(error);
@@ -894,7 +892,7 @@ void free_key_master_info(Master_info *mi)
    1 error
 */
 
-bool check_master_connection_name(LEX_STRING *name)
+bool check_master_connection_name(LEX_CSTRING *name)
 {
   if (name->length >= MAX_CONNECTION_NAME)
     return 1;
@@ -924,7 +922,7 @@ bool check_master_connection_name(LEX_STRING *name)
 
 void create_logfile_name_with_suffix(char *res_file_name, size_t length,
                                      const char *info_file, bool append,
-                                     LEX_STRING *suffix)
+                                     LEX_CSTRING *suffix)
 {
   char buff[MAX_CONNECTION_NAME+1],
     res[MAX_CONNECTION_NAME * MAX_FILENAME_MBWIDTH+1], *p;
@@ -1084,7 +1082,7 @@ bool Master_info_index::init_all_master_info()
 
   if ((index_file_nr= my_open(index_file_name,
                               O_RDWR | O_CREAT | O_BINARY ,
-                              MYF(MY_WME | ME_NOREFRESH))) < 0 ||
+                              MYF(MY_WME | ME_ERROR_LOG))) < 0 ||
       my_sync(index_file_nr, MYF(MY_WME)) ||
       init_io_cache(&index_file, index_file_nr,
                     IO_SIZE, READ_CACHE,
@@ -1117,7 +1115,7 @@ bool Master_info_index::init_all_master_info()
   while (!init_strvar_from_file(sign, sizeof(sign),
                                 &index_file, NULL))
   {
-    LEX_STRING connection_name;
+    LEX_CSTRING connection_name;
     Master_info *mi;
     char buf_master_info_file[FN_REFLEN];
     char buf_relay_log_info_file[FN_REFLEN];
@@ -1175,7 +1173,7 @@ bool Master_info_index::init_all_master_info()
     }
     else
     {
-      /* Initialization of Master_info succeded. Add it to HASH */
+      /* Initialization of Master_info succeeded. Add it to HASH */
       if (global_system_variables.log_warnings > 1)
         sql_print_information("Initialized Master_info from '%s'",
                               buf_master_info_file);
@@ -1246,7 +1244,7 @@ error:
 
 
 /* Write new master.info to master.info.index File */
-bool Master_info_index::write_master_name_to_index_file(LEX_STRING *name,
+bool Master_info_index::write_master_name_to_index_file(LEX_CSTRING *name,
                                                         bool do_sync)
 {
   DBUG_ASSERT(my_b_inited(&index_file) != 0);
@@ -1283,7 +1281,7 @@ bool Master_info_index::write_master_name_to_index_file(LEX_STRING *name,
 			WARN_LEVEL_ERROR-> Issue error if not exists
 */
 
-Master_info *get_master_info(const LEX_STRING *connection_name,
+Master_info *get_master_info(const LEX_CSTRING *connection_name,
                              Sql_condition::enum_warning_level warning)
 {
   Master_info *mi;
@@ -1300,7 +1298,7 @@ Master_info *get_master_info(const LEX_STRING *connection_name,
     if (warning != Sql_condition::WARN_LEVEL_NOTE)
       my_error(WARN_NO_MASTER_INFO,
                MYF(warning == Sql_condition::WARN_LEVEL_WARN ?
-                   ME_JUST_WARNING : 0),
+                   ME_WARNING : 0),
                (int) connection_name->length, connection_name->str);
     mysql_mutex_unlock(&LOCK_active_mi);
     DBUG_RETURN(0);
@@ -1349,12 +1347,12 @@ void Master_info::release()
 */
 
 Master_info *
-Master_info_index::get_master_info(const LEX_STRING *connection_name,
+Master_info_index::get_master_info(const LEX_CSTRING *connection_name,
                                    Sql_condition::enum_warning_level warning)
 {
   Master_info *mi;
   char buff[MAX_CONNECTION_NAME+1], *res;
-  uint buff_length;
+  size_t buff_length;
   DBUG_ENTER("get_master_info");
   DBUG_PRINT("enter",
              ("connection_name: '%.*s'", (int) connection_name->length,
@@ -1370,7 +1368,7 @@ Master_info_index::get_master_info(const LEX_STRING *connection_name,
   if (!mi && warning != Sql_condition::WARN_LEVEL_NOTE)
   {
     my_error(WARN_NO_MASTER_INFO,
-             MYF(warning == Sql_condition::WARN_LEVEL_WARN ? ME_JUST_WARNING :
+             MYF(warning == Sql_condition::WARN_LEVEL_WARN ? ME_WARNING :
                  0),
              (int) connection_name->length,
              connection_name->str);
@@ -1380,7 +1378,7 @@ Master_info_index::get_master_info(const LEX_STRING *connection_name,
 
 
 /* Check Master_host & Master_port is duplicated or not */
-bool Master_info_index::check_duplicate_master_info(LEX_STRING *name_arg,
+bool Master_info_index::check_duplicate_master_info(LEX_CSTRING *name_arg,
                                                     const char *host,
                                                     uint port)
 {
@@ -1429,7 +1427,7 @@ bool Master_info_index::add_master_info(Master_info *mi, bool write_to_file)
     We have to protect against shutdown to ensure we are not calling
     my_hash_insert() while my_hash_free() is in progress
   */
-  if (unlikely(shutdown_in_progress) ||
+  if (unlikely(abort_loop) ||
       !my_hash_insert(&master_info_hash, (uchar*) mi))
   {
     if (global_system_variables.log_warnings > 1)
@@ -1551,6 +1549,9 @@ bool give_error_if_slave_running(bool already_locked)
 /**
    any_slave_sql_running()
 
+   @param
+   already_locked  0 if we need to lock, 1 if we have LOCK_active_mi_locked
+
    @return
    0            No Slave SQL thread is running
    #		Number of slave SQL thread running
@@ -1561,26 +1562,28 @@ bool give_error_if_slave_running(bool already_locked)
    hash entries can't be accessed.
 */
 
-uint any_slave_sql_running()
+uint any_slave_sql_running(bool already_locked)
 {
   uint count= 0;
   HASH *hash;
   DBUG_ENTER("any_slave_sql_running");
 
-  mysql_mutex_lock(&LOCK_active_mi);
-  if (unlikely(shutdown_in_progress || !master_info_index))
+  if (!already_locked)
+    mysql_mutex_lock(&LOCK_active_mi);
+  if (unlikely(abort_loop || !master_info_index))
+    count= 1;
+  else
   {
+    hash= &master_info_index->master_info_hash;
+    for (uint i= 0; i< hash->records; ++i)
+    {
+      Master_info *mi= (Master_info *)my_hash_element(hash, i);
+      if (mi->rli.slave_running != MYSQL_SLAVE_NOT_RUN)
+        count++;
+    }
+  }
+  if (!already_locked)
     mysql_mutex_unlock(&LOCK_active_mi);
-    DBUG_RETURN(1);
-  }
-  hash= &master_info_index->master_info_hash;
-  for (uint i= 0; i< hash->records; ++i)
-  {
-    Master_info *mi= (Master_info *)my_hash_element(hash, i);
-    if (mi->rli.slave_running != MYSQL_SLAVE_NOT_RUN)
-      count++;
-  }
-  mysql_mutex_unlock(&LOCK_active_mi);
   DBUG_RETURN(count);
 }
 
@@ -1637,7 +1640,7 @@ bool Master_info_index::start_all_slaves(THD *thd)
     error= start_slave(thd, mi, 1);
     mi->release();
     mysql_mutex_lock(&LOCK_active_mi);
-    if (error)
+    if (unlikely(error))
     {
       my_error(ER_CANT_START_STOP_SLAVE, MYF(0),
                "START",
@@ -1710,7 +1713,7 @@ bool Master_info_index::stop_all_slaves(THD *thd)
     error= stop_slave(thd, mi, 1);
     mi->release();
     mysql_mutex_lock(&LOCK_active_mi);
-    if (error)
+    if (unlikely(error))
     {
       my_error(ER_CANT_START_STOP_SLAVE, MYF(0),
                "STOP",
@@ -1790,12 +1793,6 @@ void Domain_id_filter::reset_filter()
   m_filter= false;
 }
 
-void Domain_id_filter::clear_ids()
-{
-  reset_dynamic(&m_domain_ids[DO_DOMAIN_IDS]);
-  reset_dynamic(&m_domain_ids[IGNORE_DOMAIN_IDS]);
-}
-
 /**
   Update the do/ignore domain id filter lists.
 
@@ -1909,14 +1906,14 @@ char *Domain_id_filter::as_string(enum_list_type type)
     return NULL;
 
   // Store the total number of elements followed by the individual elements.
-  ulong cur_len= sprintf(buf, "%u", ids->elements);
+  size_t cur_len= sprintf(buf, "%u", ids->elements);
   sz-= cur_len;
 
   for (uint i= 0; i < ids->elements; i++)
   {
     ulong domain_id;
     get_dynamic(ids, (void *) &domain_id, i);
-    cur_len+= my_snprintf(buf + cur_len, sz, " %u", domain_id);
+    cur_len+= my_snprintf(buf + cur_len, sz, " %lu", domain_id);
     sz-= cur_len;
   }
   return buf;
@@ -2015,7 +2012,7 @@ bool Master_info_index::flush_all_relay_logs()
     mi->release();
     mysql_mutex_lock(&LOCK_active_mi);
 
-    if (error)
+    if (unlikely(error))
     {
       result= true;
       break;
