@@ -17,7 +17,7 @@
 /*  Include relevant sections of the System header files.              */
 /***********************************************************************/
 #include "my_global.h"
-#if !defined(_WIN32)
+#if !defined(__WIN__)
 #if defined(UNIX)
 #include <fnmatch.h>
 #include <errno.h>
@@ -27,7 +27,7 @@
 #include <io.h>
 #endif  // !UNIX
 #include <fcntl.h>
-#endif  // !_WIN32
+#endif  // !__WIN__
 #include <time.h>
 
 /***********************************************************************/
@@ -147,17 +147,17 @@ static bool ZipFile(PGLOBAL g, ZIPUTIL *zutp, PCSZ fn, PCSZ entry, char *buf)
 static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, PCSZ pat, char *buf)
 {
 	char filename[_MAX_PATH];
-	int  rc;
 
 	/*********************************************************************/
 	/*  pat is a multiple file name with wildcard characters             */
 	/*********************************************************************/
 	strcpy(filename, pat);
 
-#if defined(_WIN32)
+#if defined(__WIN__)
 	char   drive[_MAX_DRIVE], direc[_MAX_DIR];
 	WIN32_FIND_DATA FileData;
 	HANDLE hSearch;
+	int rc;
 
 	_splitpath(filename, drive, direc, NULL, NULL);
 
@@ -210,7 +210,7 @@ static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, PCSZ pat, char *buf)
 		return true;
 	} // endif FindClose
 
-#else   // !_WIN32
+#else   // !__WIN__
 	struct stat fileinfo;
 	char   fn[FN_REFLEN], direc[FN_REFLEN], pattern[FN_HEADLEN], ftype[FN_EXTLEN];
 	DIR   *dir;
@@ -251,7 +251,7 @@ static bool ZipFiles(PGLOBAL g, ZIPUTIL *zutp, PCSZ pat, char *buf)
 
 	// Close the dir handle.
 	closedir(dir);
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 	return false;
 }	// end of ZipFiles
@@ -275,9 +275,9 @@ bool ZipLoadFile(PGLOBAL g, PCSZ zfn, PCSZ fn, PCSZ entry, bool append, bool mul
 
 		if (!entry) {    // entry defaults to the file name
 			char* p = strrchr((char*)fn, '/');
-#if defined(_WIN32)
+#if defined(__WIN__)
 			if (!p) p = strrchr((char*)fn, '\\');
-#endif  //  _WIN32
+#endif  //  __WIN__
 			entp = (p) ? p + 1 : entry;
 		} else
 			entp = entry;
@@ -467,7 +467,7 @@ UNZIPUTL::UNZIPUTL(PCSZ tgt, PCSZ pw, bool mul)
 	memset(fn, 0, sizeof(fn));
 
 	// Init the case mapping table.
-#if defined(_WIN32)
+#if defined(__WIN__)
 	for (int i = 0; i < 256; ++i) mapCaseTable[i] = toupper(i);
 #else
 	for (int i = 0; i < 256; ++i) mapCaseTable[i] = i;
@@ -487,7 +487,7 @@ UNZIPUTL::UNZIPUTL(PDOSDEF tdp)
 	memset(fn, 0, sizeof(fn));
 
 	// Init the case mapping table.
-#if defined(_WIN32)
+#if defined(__WIN__)
 	for (int i = 0; i < 256; ++i) mapCaseTable[i] = toupper(i);
 #else
 	for (int i = 0; i < 256; ++i) mapCaseTable[i] = i;
@@ -769,7 +769,7 @@ bool UNZIPUTL::openEntry(PGLOBAL g)
 	} // endif rc
 
 	if (trace(1))
-		htrc("Openning entry%s %s\n", fn, (entryopen) ? "oked" : "failed");
+		htrc("Opening entry%s %s\n", fn, (entryopen) ? "oked" : "failed");
 
 	return !entryopen;
 }	// end of openEntry
