@@ -16,10 +16,9 @@ this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
-
 #ifdef WITH_WSREP
-# include <mysql/service_wsrep.h>
-# include "../../../wsrep/wsrep_api.h"
+#include "wsrep_api.h"
+#include <mysql/service_wsrep.h>
 #endif /* WITH_WSREP */
 
 #include "table.h"
@@ -59,35 +58,33 @@ struct st_handler_tablename
   const char *tablename;
 };
 /** The class defining a handle to an Innodb table */
-class ha_innobase: public handler
+class ha_innobase final: public handler
 {
 public:
 	ha_innobase(handlerton* hton, TABLE_SHARE* table_arg);
-	~ha_innobase();
+	~ha_innobase() override;
 
 	/** Get the row type from the storage engine.  If this method returns
 	ROW_TYPE_NOT_USED, the information in HA_CREATE_INFO should be used. */
-	enum row_type get_row_type() const;
+        enum row_type get_row_type() const override;
 
-	const char* table_type() const;
+        const char* table_type() const;
 
-	const char* index_type(uint key_number);
+	const char* index_type(uint key_number) override;
 
-	const char** bas_ext() const;
+	Table_flags table_flags() const override;
 
-	Table_flags table_flags() const;
+	ulong index_flags(uint idx, uint part, bool all_parts) const override;
 
-	ulong index_flags(uint idx, uint part, bool all_parts) const;
+	uint max_supported_keys() const override;
 
-	uint max_supported_keys() const;
+	uint max_supported_key_length() const override;
 
-	uint max_supported_key_length() const;
+	uint max_supported_key_part_length() const override;
 
-	uint max_supported_key_part_length() const;
+	const key_map* keys_to_use_for_scanning() override;
 
-	const key_map* keys_to_use_for_scanning();
-
-	void column_bitmaps_signal();
+	void column_bitmaps_signal() override;
 
 	/** Opens dictionary table object using table name. For partition, we need to
 	try alternative lower/upper case names to support moving data files across
@@ -103,99 +100,97 @@ public:
 		bool			is_partition,
 		dict_err_ignore_t	ignore_err);
 
-	int open(const char *name, int mode, uint test_if_locked);
+	int open(const char *name, int mode, uint test_if_locked) override;
 
-	handler* clone(const char *name, MEM_ROOT *mem_root);
+	handler* clone(const char *name, MEM_ROOT *mem_root) override;
 
-	int close(void);
+	int close(void) override;
 
-	double scan_time();
+	double scan_time() override;
 
-	double read_time(uint index, uint ranges, ha_rows rows);
+	double read_time(uint index, uint ranges, ha_rows rows) override;
 
-	longlong get_memory_buffer_size() const;
+	int delete_all_rows() override;
 
-	int delete_all_rows();
+	int write_row(const uchar * buf) override;
 
-	int write_row(uchar * buf);
+	int update_row(const uchar * old_data, const uchar * new_data) override;
 
-	int update_row(const uchar * old_data, uchar * new_data);
+	int delete_row(const uchar * buf) override;
 
-	int delete_row(const uchar * buf);
+	bool was_semi_consistent_read() override;
 
-	bool was_semi_consistent_read();
+	void try_semi_consistent_read(bool yes) override;
 
-	void try_semi_consistent_read(bool yes);
+	void unlock_row() override;
 
-	void unlock_row();
+	int index_init(uint index, bool sorted) override;
 
-	int index_init(uint index, bool sorted);
-
-	int index_end();
+	int index_end() override;
 
 	int index_read(
 		uchar*			buf,
 		const uchar*		key,
 		uint			key_len,
-		ha_rkey_function	find_flag);
+		ha_rkey_function	find_flag) override;
 
-	int index_read_last(uchar * buf, const uchar * key, uint key_len);
+	int index_read_last(uchar * buf, const uchar * key,
+			    uint key_len) override;
 
-	int index_next(uchar * buf);
+        int index_next(uchar * buf) override;
 
-	int index_next_same(uchar * buf, const uchar *key, uint keylen);
+	int index_next_same(uchar * buf, const uchar * key,
+			    uint keylen) override;
 
-	int index_prev(uchar * buf);
+	int index_prev(uchar * buf) override;
 
-	int index_first(uchar * buf);
+	int index_first(uchar * buf) override;
 
-	int index_last(uchar * buf);
+	int index_last(uchar * buf) override;
 
 	/* Copy a cached MySQL row. If requested, also avoids
 	overwriting non-read columns. */
 	void copy_cached_row(uchar *to_rec, const uchar *from_rec,
 				uint rec_length);
-	int rnd_init(bool scan);
+	int rnd_init(bool scan) override;
 
-	int rnd_end();
+	int rnd_end() override;
 
-	int rnd_next(uchar *buf);
+	int rnd_next(uchar *buf) override;
 
-	int rnd_pos(uchar * buf, uchar *pos);
+	int rnd_pos(uchar * buf, uchar *pos) override;
 
-	int ft_init();
-	void ft_end() { rnd_end(); }
-	FT_INFO *ft_init_ext(uint flags, uint inx, String* key);
-	int ft_read(uchar* buf);
+	int ft_init() override;
+	void ft_end() override { rnd_end(); }
+	FT_INFO *ft_init_ext(uint flags, uint inx, String* key) override;
+	int ft_read(uchar* buf) override;
 
-	void position(const uchar *record);
+	void position(const uchar *record) override;
 
-	int info(uint);
+	int info(uint) override;
 
-	int analyze(THD* thd,HA_CHECK_OPT* check_opt);
+	int analyze(THD* thd,HA_CHECK_OPT* check_opt) override;
 
-	int optimize(THD* thd,HA_CHECK_OPT* check_opt);
+	int optimize(THD* thd,HA_CHECK_OPT* check_opt) override;
 
-	int discard_or_import_tablespace(my_bool discard);
+	int discard_or_import_tablespace(my_bool discard) override;
 
-	int extra(ha_extra_function operation);
+	int extra(ha_extra_function operation) override;
 
-	int reset();
+	int reset() override;
 
-	int external_lock(THD *thd, int lock_type);
+	int external_lock(THD *thd, int lock_type) override;
 
-	int start_stmt(THD *thd, thr_lock_type lock_type);
-
-	void position(uchar *record);
+	int start_stmt(THD *thd, thr_lock_type lock_type) override;
 
 	ha_rows records_in_range(
 		uint			inx,
 		key_range*		min_key,
-		key_range*		max_key);
+		key_range*		max_key) override;
 
-	ha_rows estimate_rows_upper_bound();
+	ha_rows estimate_rows_upper_bound() override;
 
-	void update_create_info(HA_CREATE_INFO* create_info);
+	void update_create_info(HA_CREATE_INFO* create_info) override;
 
 	inline int create(
 		const char*		name,
@@ -207,77 +202,71 @@ public:
 	int create(
 		const char*		name,
 		TABLE*			form,
-		HA_CREATE_INFO*		create_info);
-
-	const char* check_table_options(THD *thd, TABLE* table,
-		HA_CREATE_INFO*	create_info, const bool use_tablespace, const ulint file_format);
+		HA_CREATE_INFO*		create_info) override;
 
 	inline int delete_table(const char* name, enum_sql_command sqlcom);
 
-	int truncate();
+	int truncate() override;
 
-	int delete_table(const char *name);
+	int delete_table(const char *name) override;
 
-	int rename_table(const char* from, const char* to);
+	int rename_table(const char* from, const char* to) override;
 	int defragment_table(const char* name, const char* index_name,
 						bool async);
-	int check(THD* thd, HA_CHECK_OPT* check_opt);
-	char* update_table_comment(const char* comment);
+	int check(THD* thd, HA_CHECK_OPT* check_opt) override;
+	char* update_table_comment(const char* comment) override;
 
-	char* get_foreign_key_create_info();
+	char* get_foreign_key_create_info() override;
 
-	int get_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
+        int get_foreign_key_list(THD *thd,
+                                 List<FOREIGN_KEY_INFO> *f_key_list) override;
 
 	int get_parent_foreign_key_list(
 		THD*			thd,
-		List<FOREIGN_KEY_INFO>*	f_key_list);
-	int get_cascade_foreign_key_table_list(
-		THD*				thd,
-		List<st_handler_tablename>*	fk_table_list);
+		List<FOREIGN_KEY_INFO>*	f_key_list) override;
 
+	bool can_switch_engines() override;
 
-	bool can_switch_engines();
+	uint referenced_by_foreign_key() override;
 
-	uint referenced_by_foreign_key();
+	void free_foreign_key_create_info(char* str) override;
 
-	void free_foreign_key_create_info(char* str);
-
-	uint lock_count(void) const;
+	uint lock_count(void) const override;
 
 	THR_LOCK_DATA** store_lock(
 		THD*			thd,
 		THR_LOCK_DATA**		to,
-		thr_lock_type		lock_type);
+		thr_lock_type		lock_type) override;
 
-	void init_table_handle_for_HANDLER();
+	void init_table_handle_for_HANDLER() override;
 
-	virtual void get_auto_increment(
+	void get_auto_increment(
 		ulonglong		offset,
 		ulonglong		increment,
 		ulonglong		nb_desired_values,
 		ulonglong*		first_value,
-		ulonglong*		nb_reserved_values);
-	int reset_auto_increment(ulonglong value);
+		ulonglong*		nb_reserved_values) override;
+	int reset_auto_increment(ulonglong value) override;
 
-	virtual bool get_error_message(int error, String *buf);
+	bool get_error_message(int error, String *buf) override;
 
-	virtual bool get_foreign_dup_key(char*, uint, char*, uint);
+	bool get_foreign_dup_key(char*, uint, char*, uint) override;
 
-	uint8 table_cache_type();
+	uint8 table_cache_type() override;
 
 	/**
 	Ask handler about permission to cache table during query registration
 	*/
 	my_bool register_query_cache_table(
 		THD*			thd,
-		char*			table_key,
+		const char*		table_key,
 		uint			key_length,
 		qc_engine_callback*	call_back,
-		ulonglong*		engine_data);
+		ulonglong*		engine_data) override;
 
-	bool primary_key_is_clustered();
+	bool primary_key_is_clustered() override;
 
-	int cmp_ref(const uchar* ref1, const uchar* ref2);
+	int cmp_ref(const uchar* ref1, const uchar* ref2) override;
 
 	/** On-line ALTER TABLE interface @see handler0alter.cc @{ */
 
@@ -287,15 +276,27 @@ public:
 	by ALTER TABLE and holding data used during in-place alter.
 
 	@retval HA_ALTER_INPLACE_NOT_SUPPORTED Not supported
-	@retval HA_ALTER_INPLACE_NO_LOCK Supported
-	@retval HA_ALTER_INPLACE_SHARED_LOCK_AFTER_PREPARE
-		Supported, but requires lock during main phase and
-		exclusive lock during prepare phase.
-	@retval HA_ALTER_INPLACE_NO_LOCK_AFTER_PREPARE
-		Supported, prepare phase requires exclusive lock.  */
+	@retval HA_ALTER_INPLACE_INSTANT
+	MDL_EXCLUSIVE is needed for executing prepare_inplace_alter_table()
+	and commit_inplace_alter_table(). inplace_alter_table()
+	will not be called.
+	@retval HA_ALTER_INPLACE_COPY_NO_LOCK
+	MDL_EXCLUSIVE in prepare_inplace_alter_table(), which can be downgraded
+	to LOCK=NONE for rebuilding the table in inplace_alter_table()
+	@retval HA_ALTER_INPLACE_COPY_LOCK
+	MDL_EXCLUSIVE in prepare_inplace_alter_table(), which can be downgraded
+	to LOCK=SHARED for rebuilding the table in inplace_alter_table()
+	@retval HA_ALTER_INPLACE_NOCOPY_NO_LOCK
+	MDL_EXCLUSIVE in prepare_inplace_alter_table(), which can be downgraded
+	to LOCK=NONE for inplace_alter_table() which will not rebuild the table
+	@retval HA_ALTER_INPLACE_NOCOPY_LOCK
+	MDL_EXCLUSIVE in prepare_inplace_alter_table(), which can be downgraded
+	to LOCK=SHARED for inplace_alter_table() which will not rebuild
+	the table. */
+
 	enum_alter_inplace_result check_if_supported_inplace_alter(
 		TABLE*			altered_table,
-		Alter_inplace_info*	ha_alter_info);
+		Alter_inplace_info*	ha_alter_info) override;
 
 	/** Allows InnoDB to update internal structures with concurrent
 	writes blocked (provided that check_if_supported_inplace_alter()
@@ -311,7 +312,7 @@ public:
 	*/
 	bool prepare_inplace_alter_table(
 		TABLE*			altered_table,
-		Alter_inplace_info*	ha_alter_info);
+		Alter_inplace_info*	ha_alter_info) override;
 
 	/** Alter the table structure in-place with operations
 	specified using HA_ALTER_FLAGS and Alter_inplace_information.
@@ -327,7 +328,7 @@ public:
 	*/
 	bool inplace_alter_table(
 		TABLE*			altered_table,
-		Alter_inplace_info*	ha_alter_info);
+		Alter_inplace_info*	ha_alter_info) override;
 
 	/** Commit or rollback the changes made during
 	prepare_inplace_alter_table() and inplace_alter_table() inside
@@ -346,12 +347,12 @@ public:
 	bool commit_inplace_alter_table(
 		TABLE*			altered_table,
 		Alter_inplace_info*	ha_alter_info,
-		bool			commit);
+		bool			commit) override;
 	/** @} */
 
 	bool check_if_incompatible_data(
 		HA_CREATE_INFO*		info,
-		uint			table_changes);
+		uint			table_changes) override;
 
 	/** @name Multi Range Read interface @{ */
 
@@ -366,11 +367,11 @@ public:
 		void*			seq_init_param,
 		uint			n_ranges,
 		uint			mode,
-		HANDLER_BUFFER*		buf);
+		HANDLER_BUFFER*		buf) override;
 
 	/** Process next multi range read @see DsMrr_impl::dsmrr_next
 	@param range_info */
-	int multi_range_read_next(range_id_t *range_info);
+	int multi_range_read_next(range_id_t *range_info) override;
 
 	/** Initialize multi range read and get information.
 	@see ha_myisam::multi_range_read_info_const
@@ -389,7 +390,7 @@ public:
 		uint			n_ranges,
 		uint*			bufsz,
 		uint*			flags,
-		Cost_estimate*		cost);
+		Cost_estimate*		cost) override;
 
 	/** Initialize multi range read and get information.
 	@see DsMrr_impl::dsmrr_info
@@ -402,16 +403,16 @@ public:
 	@param cost */
 	ha_rows multi_range_read_info(uint keyno, uint n_ranges, uint keys,
 				      uint key_parts, uint* bufsz, uint* flags,
-				      Cost_estimate* cost);
+				      Cost_estimate* cost) override;
 
 	int multi_range_read_explain_info(uint mrr_mode,
-					  char *str, size_t size);
+					  char *str, size_t size) override;
 
 	/** Attempt to push down an index condition.
 	@param[in] keyno MySQL key number
 	@param[in] idx_cond Index condition to be checked
 	@return idx_cond if pushed; NULL if not pushed */
-	Item* idx_cond_push(uint keyno, Item* idx_cond);
+	Item* idx_cond_push(uint keyno, Item* idx_cond) override;
 	/* @} */
 
 	/** Check if InnoDB is not storing virtual column metadata for a table.
@@ -422,13 +423,35 @@ public:
 		return s.frm_version<FRM_VER_EXPRESSSIONS && s.virtual_fields;
 	}
 
-protected:
-	/**
-	MySQL calls this method at the end of each statement. This method
-	exists for readability only, called from reset(). The name reset()
-	doesn't give any clue that it is called at the end of a statement. */
-	int end_stmt();
+	/** Push a primary key filter.
+	@param[in]	pk_filter	filter against which primary keys
+					are to be checked
+	@retval	false if pushed (always) */
+	bool rowid_filter_push(Rowid_filter *rowid_filter) override;
 
+	bool
+	can_convert_string(const Field_string* field,
+			   const Column_definition& new_field) const override;
+	bool can_convert_varstring(
+	    const Field_varstring* field,
+	    const Column_definition& new_field) const override;
+	bool
+	can_convert_blob(const Field_blob* field,
+			 const Column_definition& new_field) const override;
+
+	/** @return whether innodb_strict_mode is active */
+	static bool is_innodb_strict_mode(THD* thd);
+
+	/** @return whether innodb_strict_mode is active */
+	bool is_innodb_strict_mode()
+	{ return is_innodb_strict_mode(m_user_thd); }
+	Compare_keys
+	compare_key_parts(const Field& old_field,
+			  const Column_definition& new_field,
+			  const KEY_PART_INFO& old_part,
+			  const KEY_PART_INFO& new_part) const override;
+
+protected:
 	dberr_t innobase_get_autoinc(ulonglong* value);
 	dberr_t innobase_lock_autoinc();
 	ulonglong innobase_peek_autoinc();
@@ -447,8 +470,11 @@ protected:
 	dict_index_t* innobase_get_index(uint keynr);
 
 #ifdef WITH_WSREP
-	int wsrep_append_keys(THD *thd, wsrep_key_type key_type,
-			      const uchar* record0, const uchar* record1);
+	int wsrep_append_keys(
+		THD *thd,
+		Wsrep_service_key_type key_type,
+		const uchar* record0,
+		const uchar* record1);
 #endif
 	/** Builds a 'template' to the prebuilt struct.
 
@@ -458,7 +484,7 @@ protected:
 	false if accessing individual fields is enough */
 	void build_template(bool whole_row);
 
-	virtual int info_low(uint, bool);
+	int info_low(uint, bool);
 
 	/** The multi range read session object */
 	DsMrr_impl		m_ds_mrr;
@@ -503,13 +529,8 @@ the definitions are bracketed with #ifdef INNODB_COMPATIBILITY_HOOKS */
 #error InnoDB needs MySQL to be built with #define INNODB_COMPATIBILITY_HOOKS
 #endif
 
-LEX_STRING* thd_query_string(MYSQL_THD thd);
-size_t thd_query_safe(MYSQL_THD thd, char *buf, size_t buflen);
-
 extern "C" {
 
-struct charset_info_st *thd_charset(MYSQL_THD thd);
-
 /** Check if a user thread is a replication slave thread
 @param thd user thread
 @retval 0 the user thread is not a replication slave thread
@@ -560,23 +581,8 @@ extern void mysql_bin_log_commit_pos(THD *thd, ulonglong *out_pos, const char **
 
 struct trx_t;
 #ifdef WITH_WSREP
-//extern "C" int wsrep_trx_order_before(void *thd1, void *thd2);
-
-extern "C" bool wsrep_thd_is_wsrep_on(THD *thd);
-
-
-extern "C" void wsrep_thd_set_exec_mode(THD *thd, enum wsrep_exec_mode mode);
-extern "C" void wsrep_thd_set_query_state(
-	THD *thd, enum wsrep_query_state state);
-
-extern "C" void wsrep_thd_set_trx_to_replay(THD *thd, uint64 trx_id);
-
-extern "C" uint32 wsrep_thd_wsrep_rand(THD *thd);
-extern "C" time_t wsrep_thd_query_start(THD *thd);
-extern "C" query_id_t wsrep_thd_query_id(THD *thd);
-extern "C" query_id_t wsrep_thd_wsrep_last_query_id(THD *thd);
-extern "C" void wsrep_thd_set_wsrep_last_query_id(THD *thd, query_id_t id);
-#endif
+#include <mysql/service_wsrep.h>
+#endif /* WITH_WSREP */
 
 extern const struct _ft_vft ft_vft_result;
 
@@ -760,6 +766,8 @@ private:
 
 	/** Table name */
 	char*		m_table_name;
+	/** Table */
+	dict_table_t*	m_table;
 	/** Whether the table needs to be dropped before rollback */
 	bool		m_drop_before_rollback;
 
@@ -873,19 +881,6 @@ innodb_base_col_setup_for_stored(
 	create_table_info_t::normalize_table_name_low(norm_name, name, FALSE)
 #endif /* _WIN32 */
 
-/** Converts an InnoDB error code to a MySQL error code.
-Also tells to MySQL about a possible transaction rollback inside InnoDB caused
-by a lock wait timeout or a deadlock.
-@param[in]	error	InnoDB error code.
-@param[in]	flags	InnoDB table flags or 0.
-@param[in]	thd	MySQL thread or NULL.
-@return MySQL error code */
-int
-convert_error_code_to_mysql(
-	dberr_t	error,
-	ulint	flags,
-	THD*	thd);
-
 /** Converts a search mode flag understood by MySQL to a flag understood
 by InnoDB.
 @param[in]	find_flag	MySQL search mode flag.
