@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Percona Inc.
-Copyright (c) 2013, 2020, MariaDB Corporation.
+Copyright (c) 2013, 2019, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted
 by Percona Inc.. Those modifications are
@@ -36,7 +36,7 @@ Created 10/21/1995 Heikki Tuuri
 #ifndef os0file_h
 #define os0file_h
 
-#include "page0size.h"
+#include "fsp0types.h"
 #include "os0api.h"
 
 #ifndef _WIN32
@@ -151,7 +151,6 @@ static const ulint OS_FILE_NORMAL = 62;
 static const ulint OS_DATA_FILE = 100;
 static const ulint OS_LOG_FILE = 101;
 static const ulint OS_DATA_TEMP_FILE = 102;
-static const ulint OS_DATA_FILE_NO_O_DIRECT = 103;
 /* @} */
 
 /** Error codes from os_file_get_last_error @{ */
@@ -361,17 +360,8 @@ public:
 
 	/** Set the pointer to file node for IO
 	@param[in] node			File node */
-	void set_fil_node(fil_node_t* node)
-	{
-		if (node && !fil_node_should_punch_hole(node)) {
-			clear_punch_hole();
-		}
+	inline void set_fil_node(fil_node_t* node);
 
-		m_fil_node = node;
-	}
-
-	/** Compare two requests
-	@reutrn true if the are equal */
 	bool operator==(const IORequest& rhs) const
 	{
 		return(m_type == rhs.m_type);
@@ -415,17 +405,7 @@ public:
 			: 0);
 	}
 
-	bool should_punch_hole() const {
-		return (m_fil_node ?
-			fil_node_should_punch_hole(m_fil_node)
-			: false);
-	}
-
-	void space_no_punch_hole() const {
-		if (m_fil_node) {
-			fil_space_set_punch_hole(m_fil_node, false);
-		}
-	}
+	inline bool should_punch_hole() const;
 
 	/** Free storage space associated with a section of the file.
 	@param[in]	fh		Open file handle
@@ -1586,19 +1566,6 @@ os_file_change_size_win32(
 
 #endif /*_WIN32 */
 
-/** Check if the file system supports sparse files.
-
-Warning: On POSIX systems we try and punch a hole from offset 0 to
-the system configured page size. This should only be called on an empty
-file.
-
-@param[in]	fh		File handle for the file - if opened
-@return true if the file system supports sparse files */
-bool
-os_is_sparse_file_supported(
-	os_file_t	fh)
-	MY_ATTRIBUTE((warn_unused_result));
-
 /** Free storage space associated with a section of the file.
 @param[in]	fh		Open file handle
 @param[in]	off		Starting offset (SEEK_SET)
@@ -1638,16 +1605,6 @@ is_absolute_path(
 	return(false);
 }
 
-/***********************************************************************//**
-Try to get number of bytes per sector from file system.
-@return	file block size */
-UNIV_INTERN
-ulint
-os_file_get_block_size(
-/*===================*/
-	os_file_t	file,	/*!< in: handle to a file */
-	const char*	name);	/*!< in: file name */
-
 #include "os0file.ic"
 
 #endif /* os0file_h */
