@@ -29,8 +29,8 @@ Created 2/2/1994 Heikki Tuuri
 #include "page0cur.h"
 #include "page0zip.h"
 #include "buf0buf.h"
+#include "buf0checksum.h"
 #include "btr0btr.h"
-#include "row0trunc.h"
 #include "srv0srv.h"
 #include "lock0lock.h"
 #include "fut0lst.h"
@@ -449,28 +449,19 @@ page_create_zip(
 	buf_block_t*		block,		/*!< in/out: a buffer frame
 						where the page is created */
 	dict_index_t*		index,		/*!< in: the index of the
-						page, or NULL when applying
-						TRUNCATE log
-						record during recovery */
+						page */
 	ulint			level,		/*!< in: the B-tree level
 						of the page */
 	trx_id_t		max_trx_id,	/*!< in: PAGE_MAX_TRX_ID */
-	const redo_page_compress_t* page_comp_info,
-						/*!< in: used for applying
-						TRUNCATE log
-						record during recovery */
 	mtr_t*			mtr)		/*!< in/out: mini-transaction
 						handle */
 {
 	page_t*			page;
 	page_zip_des_t*		page_zip = buf_block_get_page_zip(block);
-	bool			is_spatial;
 
 	ut_ad(block);
 	ut_ad(page_zip);
-	ut_ad(index == NULL || dict_table_is_comp(index->table));
-	is_spatial = index ? dict_index_is_spatial(index)
-			   : page_comp_info->type & DICT_SPATIAL;
+	ut_ad(dict_table_is_comp(index->table));
 
 	/* PAGE_MAX_TRX_ID or PAGE_ROOT_AUTO_INC are always 0 for
 	temporary tables. */
@@ -488,22 +479,11 @@ page_create_zip(
 	      || !dict_index_is_sec_or_ibuf(index)
 	      || index->table->is_temporary());
 
-	page = page_create_low(block, TRUE, is_spatial);
+	page = page_create_low(block, TRUE, dict_index_is_spatial(index));
 	mach_write_to_2(PAGE_HEADER + PAGE_LEVEL + page, level);
 	mach_write_to_8(PAGE_HEADER + PAGE_MAX_TRX_ID + page, max_trx_id);
 
-	if (truncate_t::s_fix_up_active) {
-		/* Compress the index page created when applying
-		TRUNCATE log during recovery */
-		if (!page_zip_compress(page_zip, page, index, page_zip_level,
-				       page_comp_info, NULL)) {
-			/* The compression of a newly created
-			page should always succeed. */
-			ut_error;
-		}
-
-	} else if (!page_zip_compress(page_zip, page, index,
-				      page_zip_level, NULL, mtr)) {
+	if (!page_zip_compress(page_zip, page, index, page_zip_level, mtr)) {
 		/* The compression of a newly created
 		page should always succeed. */
 		ut_error;
@@ -549,7 +529,7 @@ page_create_empty(
 		ut_ad(!index->table->is_temporary());
 		page_create_zip(block, index,
 				page_header_get_field(page, PAGE_LEVEL),
-				max_trx_id, NULL, mtr);
+				max_trx_id, mtr);
 	} else {
 		page_create(block, mtr, page_is_comp(page),
 			    dict_index_is_spatial(index));
@@ -724,11 +704,8 @@ page_copy_rec_list_end(
 	if (new_page_zip) {
 		mtr_set_log_mode(mtr, log_mode);
 
-		if (!page_zip_compress(new_page_zip,
-				       new_page,
-				       index,
-				       page_zip_level,
-				       NULL, mtr)) {
+		if (!page_zip_compress(new_page_zip, new_page, index,
+				       page_zip_level, mtr)) {
 			/* Before trying to reorganize the page,
 			store the number of preceding records on the page. */
 			ulint	ret_pos
@@ -893,7 +870,7 @@ page_copy_rec_list_start(
 				goto zip_reorganize;);
 
 		if (!page_zip_compress(new_page_zip, new_page, index,
-				       page_zip_level, NULL, mtr)) {
+				       page_zip_level, mtr)) {
 			ulint	ret_pos;
 #ifndef DBUG_OFF
 zip_reorganize:
@@ -1834,6 +1811,7 @@ page_print_list(
 	count = 0;
 	for (;;) {
 		offsets = rec_get_offsets(cur.rec, index, offsets,
+					  page_rec_is_leaf(cur.rec),
 					  ULINT_UNDEFINED, &heap);
 		page_rec_print(cur.rec, offsets);
 
@@ -1856,6 +1834,7 @@ page_print_list(
 
 		if (count + pr_n >= n_recs) {
 			offsets = rec_get_offsets(cur.rec, index, offsets,
+						  page_rec_is_leaf(cur.rec),
 						  ULINT_UNDEFINED, &heap);
 			page_rec_print(cur.rec, offsets);
 		}
@@ -2549,16 +2528,14 @@ wrong_page_type:
 					ib::error() << "REC_INFO_MIN_REC_FLAG "
 						"is set in a leaf-page record";
 					ret = false;
-				} else if (rec_get_deleted_flag(
-						   rec, page_is_comp(page))) {
-					/* If this were a 10.4 metadata
-					record for index->table->instant
-					we should not get here in 10.3, because
-					the metadata record should not have
-					been recognized by
-					btr_cur_instant_init_low(). */
-					ib::error() << "Metadata record "
-						"is delete-marked";
+				} else if (!rec_get_deleted_flag(
+						   rec, page_is_comp(page))
+					   != !index->table->instant) {
+					ib::error() << (index->table->instant
+							? "Metadata record "
+							"is not delete-marked"
+							: "Metadata record "
+							"is delete-marked");
 					ret = false;
 				}
 			} else if (!page_has_prev(page)
